<?php

namespace Tests\Unit;

use Tests\TestCase;
use Illuminate\Support\Facades\File;
use Illuminate\Foundation\Testing\WithFaker;
use Illuminate\Foundation\Testing\RefreshDatabase;

use Goldenscarab\Modulus\Service\Field\Facades\Field;

/**
 *  ./vendor/bin/phpunit ./vendor/goldenscarab/modulus-service-field/tests/FieldsTest.php
 */
class FieldsTest extends TestCase
{

    private $data_source = [
    	'type' => [
			'id'    => 'mon-id',
			'label' => 'mon-label',
			'code'  => 'mon-code'
    	]
    ];

    private $options = [
    	1 => 'Option 1',
    	2 => 'Option 2',
    	3 => 'Option 3',
    	4 => 'Option 4',
    	5 => 'Option 5'
    ];

    private $collection = [
    	['id' => 1, 'title' => 'Titre 1', 'value' => 'value-1', 'data' => ['data1' => 'data-value1.1', 'data2' => 'data-value1.2']],
    	['id' => 2, 'title' => 'Titre 2', 'value' => 'value-2', 'data' => ['data1' => 'data-value2.1', 'data2' => 'data-value2.2']],
    	['id' => 3, 'title' => 'Titre 3', 'value' => 'value-3', 'data' => ['data1' => 'data-value3.1', 'data2' => 'data-value3.2']],
    	['id' => 4, 'title' => 'Titre 4', 'value' => 'value-4', 'data' => ['data1' => 'data-value4.1', 'data2' => 'data-value4.2']]
    ];

    private $lorem = 'Lorem ipsum dolor sit amet, consectetur adipisicing elit. Laboriosam, rem esse itaque inventore provident reiciendis quo, asperiores dolorem voluptate nostrum quisquam similique commodi quam consequatur ad eligendi iusto aperiam quos?';



    /**
     * Test du fonctionnement de la classe en mode test unitaire
     * @return void
     */
    public function testTest()
    {
        $this->assertTrue(true);
    }

    public function testInputField()
    {
        $generated_string = Field::input([
            'type'   => 'text',
            'label'  => 'Total',
            'name'   => 'total',
            'value'  => 23.4,
        ]);

        $desired_string = File::get(__DIR__ . '/html/fields/input.html');

        $this->assertEquals($desired_string, $generated_string);
    }

    public function testInputAdvancedField()
    {
        $generated_string = Field::input([
			'id'              => 'total-final',
			'type'            => 'text',
			'label'           => 'Total',
			'name'            => 'total',
			'value'           => 23.4,
			'class'           => 'text-danger',
			'prefix'          => '<i class="fa fa-money"></i>',
			'suffix'          => '€',
			'placeholder'     => "123,30",
			'help'            => 'Taxe comprise',
			'indice'          => 1,
			'style'           => 'background-color: #c3c3c3;',
			'attributes'      => [
				'onchange' => 'console.log("change")',
		        'title' => [
		            'source'     => $this->data_source,
		            'template'   => '%s ~ %s',
		            'attributes' => ['type.id', 'type.label'],
		            'callback'   => 'ucfirst'
		        ]
			],
			'datas' => [
		        'type'      => 'column',
		        'reference' => [
		            'source'     => $this->data_source,
		            'template'   => '%s|%s',
		            'attributes' => ['type.id', 'type.code'],
		            'callback'   => 'ucfirst'
		        ]
		    ],
			'is_inline'       => true,
			'is_disabled'     => true,
			'is_readonly'     => true,
			'is_required'     => true,
			'is_autocomplete' => true,
			'is_autofocus'    => true,
		    'popover'     => [
		        'placement' => 'right',
		        'title'     => 'Information',
		        'content'   => "Le montant est calculé selon les éléments communiqué. Il peux néanmoins variée suivant d'autre facteurs non pris en compte"
		    ],
		]);

        $desired_string = File::get(__DIR__ . '/html/fields/input_advanced.html');

        $this->assertEquals($desired_string, $generated_string);
    }

    public function testSelectSimpleField()
    {
        $generated_string = Field::select([
			'label'       => 'Collection',
			'name'        => 'collection',
			'value'       => 4,
			'class'       => 'select2',
			'prefix'      => '<i class="fa fa-list"></i>',
			'suffix'      => '<i class="fa fa-hand-paper-o"></i>',
			'size'        => 'sm',
			'indice'      => 34,
			'is_required' => true,
			'options'     => $this->options
		]);

        $desired_string = File::get(__DIR__ . '/html/fields/select_simple.html');

        $this->assertEquals($desired_string, $generated_string);
    }

    public function testSelectGroupField()
    {
        $generated_string = Field::select([
			'label'       => 'Menu déroulant',
			'name'        => 'my_field_name',
			'value'       => 23,
			'placeholder' => 'Choisir ...',
			'size'        => 'sm',
			'is_required' => true,
			'options' => [
				'Groupe 1' => [
					23 => 'Élement 23',
					24 => 'Élement 24',
					25 => 'Élement 25',
				],
				'Groupe 2' => [
					26 => 'Élement 26',
					27 => 'Élement 27',
					28 => 'Élement 28',
				]
			]
		]);

        $desired_string = File::get(__DIR__ . '/html/fields/select_group.html');

        $this->assertEquals($desired_string, $generated_string);
    }

    public function testSelectMultipleField()
    {
        $generated_string = Field::select([
			'label'       => 'Collection',
			'name'        => 'collection',
			'values'      => [2, 4],
			'class'       => 'select2',
			'prefix'      => '<i class="fa fa-list"></i>',
			'suffix'      => '<i class="fa fa-hand-paper-o"></i>',
			'indice'      => 34,
			'is_required' => true,
			'is_multiple' => true,
			'options' => [
		        'collect'  => $this->collection,
		        'value'    => 'id',
		        'template' => '%s - %s',
		        'callback' => 'strtoupper',
		        'text'     => ['title', 'value'],
		    ]
		]);

        $desired_string = File::get(__DIR__ . '/html/fields/select_multiple.html');

        $this->assertEquals($desired_string, $generated_string);
    }

    public function testSelectDataField()
    {
    	$generated_string = Field::select([
			'label'       => 'Ville',
			'name'        => 'city_id',
			'value'       => 2,
			'placeholder' => "Choisir...",
			'options' => [
		        'collect'  => $this->collection,
		        'value'    => 'id',
		        'text'     => ['title', 'value'],
		        'template' => '%s - %s',
		        'datas'    => [
		            'data1' => 'data.data1',
		            'data2' => 'data.data2',
		        ]
		    ]
		]);

        $desired_string = File::get(__DIR__ . '/html/fields/select_data.html');

        $this->assertEquals($desired_string, $generated_string);
    }

    public function testTextareaField()
    {
    	$generated_string = Field::textarea([
			'label'   => 'Contenu',
			'name'    => 'content',
			'value'   => $this->lorem,
			'class'   => 'text-editor',
			'rows'    => 15,
			'popover' => [
		        'placement' => 'right',
		        'title'     => 'Le contenu',
		        'content'   => "Le contenu est le coeur de l'article. Grâce à l'éditeur vous pouvez formater le rendu selon votre goût."
		    ]
		]);

		$desired_string = File::get(__DIR__ . '/html/fields/textarea.html');

		$this->assertEquals($desired_string, $generated_string);
    }

    public function testFileField()
    {
    	$generated_string = Field::file([
			'label'       => 'Uploader',
			'name'        => 'csv',
			'prefix'      => 'Fichier',
			'placeholder' => 'Aucun fichier choisi'
		]);

		$desired_string = File::get(__DIR__ . '/html/fields/file.html');

		$this->assertEquals($desired_string, $generated_string);
    }

    public function testPdfField()
    {
    	$generated_string = Field::pdf([
			'label'       => 'Champs pdf',
			'name'        => 'my_field_pdf',
			'value'       => 'test.pdf',
			'placeholder' => 'ex: /upload/folder/file.pdf',
			'suffix'      => '<i class="fa fa-file-pdf-o mr-2"></i> Choisir un PDF',
			'is_required' => true,
		]);

		$desired_string = File::get(__DIR__ . '/html/fields/pdf.html');

		$this->assertEquals($desired_string, $generated_string);
    }

    public function testVideoField()
    {
    	$generated_string = Field::pdf([
			'label'       => 'Champs video',
			'name'        => 'my_field_video',
			'value'       => 'my-video.mp4',
			'placeholder' => 'ex: /upload/folder/video.mp4',
			'suffix'      => '<i class="fa fa-file-video-o mr-2"></i> Choisir une vidéo',
			'is_required' => true,
		]);

		$desired_string = File::get(__DIR__ . '/html/fields/video.html');

		$this->assertEquals($desired_string, $generated_string);
    }

    public function testImageField()
    {
    	$generated_string = Field::pdf([
			'label'       => 'Champs image',
			'name'        => 'my_field_image',
			'value'       => 'image.png',
			'placeholder' => 'ex: /upload/folder/image.png',
			'suffix'      => '<i class="fa fa-file-image-o mr-2"></i> Choisir une image',
			'is_required' => true,
		]);

		$desired_string = File::get(__DIR__ . '/html/fields/image.html');

		$this->assertEquals($desired_string, $generated_string);
    }

    public function testToogleField()
    {
    	$generated_string = Field::toggle([
			'label' => 'Toggle',
			'name'  => 'toggle',
			'value' => 1,
			'datas' => [
				'on'       => 'Actif',
				'off'      => 'Inactif',
				'onstyle'  => 'success',
				'offstyle' => 'danger',
				'toggle'   => 'toggle',
				'width'    => '75',
				'size'     => 'xs'
			]
		]);

		$desired_string = File::get(__DIR__ . '/html/fields/toggle.html');

		$this->assertEquals($desired_string, $generated_string);
    }

    public function testRangeField()
    {
    	$generated_string = Field::range([
			'label' => 'Zoom',
			'name'  => 'zoom',
			'min'   => 0,
			'max'   => 20,
			'value' => 7
		]);

		$desired_string = File::get(__DIR__ . '/html/fields/range.html');

		$this->assertEquals($desired_string, $generated_string);
    }

    public function testCodeField()
    {
    	$generated_string = Field::code([
			'label'       => 'Contenu',
			'name'        => 'content',
			'value'       => 'function foo(items) {
    var x = "All this is syntax highlighted";
    return x;
}',
			'style'       => 'min-height: 400px;',
			'is_required' => true,
			'datas' => array(
				'language'  => 'javascript',
				'readonly'  => 'false',
				'beautiful' => 'false',
			)
		]);

		$desired_string = File::get(__DIR__ . '/html/fields/code.html');

		$this->assertEquals($desired_string, $generated_string);
    }

    public function testRadioField()
    {
    	$generated_string = Field::radio([
			'label'     => 'Choix unique',
			'name'      => 'my_field_name',
			'value'     => 2,
			'is_inline' => true,
		    'options' => $this->options
		]);

		$desired_string = File::get(__DIR__ . '/html/fields/radio.html');

		$this->assertEquals($desired_string, $generated_string);
    }

    public function testRadioIndiceTrueField()
    {
        $generated_string = Field::radio([
            'label'     => 'Choix unique',
            'name'      => 'my_field_name',
            'value'     => 2,
            'indice'    => true,
            'is_inline' => true,
            'options' => $this->options
        ]);

        $desired_string = File::get(__DIR__ . '/html/fields/radio_indice_true.html');

        $this->assertEquals($desired_string, $generated_string);
    }

    public function testRadioCollectField()
    {
        $generated_string = Field::radio([
            'label'     => 'Mon Champs',
            'name'      => 'mon-champs',
            'value'     =>  'value-2',
            'is_inline' => false,
            'options'   => [
                'collect'  => $this->collection,
                'value'    => 'value',
                'template' => '%s - %s',
                'text'     => ['id', 'title'],
                'callback' => 'strtoupper',
                'datas' => [
                    'one' => 'data.data1',
		            'tow' => 'data.data2',
                ]
            ]
        ]);

        $desired_string = File::get(__DIR__ . '/html/fields/radio_collect.html');

        $this->assertEquals($desired_string, $generated_string);
    }

    public function testCheckboxField()
    {
    	$generated_string = Field::checkbox([
			'label'     => 'Choix multiple',
			'name'      => 'my_field_name',
			'values'    => [2, 3],
			'is_inline' => true,
		    'options' => $this->options
		]);

		$desired_string = File::get(__DIR__ . '/html/fields/checkbox.html');

		$this->assertEquals($desired_string, $generated_string);
    }

    public function testCheckboxIndiceField()
    {
        $generated_string = Field::checkbox([
            'label'     => 'Choix multiple',
            'name'      => 'my_field_name',
            'indice'    => 12,
            'values'    => [2, 3],
            'is_inline' => true,
            'options' => $this->options
        ]);

        $desired_string = File::get(__DIR__ . '/html/fields/checkbox_indice.html');

        $this->assertEquals($desired_string, $generated_string);
    }

    public function testCheckboxCollectField()
    {
        $generated_string = Field::checkbox([
            'label'     => 'Mon Champs',
            'name'      => 'mon-champs',
            'values'    => ['value-2', 'value-3'],
            'is_inline' => false,
            'options'   => [
                'collect'  => $this->collection,
                'value'    => 'value',
                'template' => '%s - %s',
                'text'     => ['id', 'title'],
                'callback' => 'strtoupper',
                'datas' => [
                    'one' => 'data.data1',
		            'tow' => 'data.data2',
                ]
            ]
        ]);

        $desired_string = File::get(__DIR__ . '/html/fields/checkbox_collect.html');

        $this->assertEquals($desired_string, $generated_string);
    }

}
